/*****************************************************************************************
 * Setup Module
 * Called when the init switch is pressed in order to load all parameters 
 * Used only once for setting up the board
 * 
 * Web interface is available through ssid = BirdFeeder-Access-Point 
 */

#include "Setup.h"
#include "Eeprom.h"
#include "Firebase.h"

// Set web server port number to 80
AsyncWebServer server(80);

void Setup_getCredentials() {


  boolean paramCaptured = false;
  
  // Start wifi in access point mode
  //
  Serial.println("Entering Access Point Mode");
  setUpWiFiAccessPoint();       
     
  // Send web page for capturing parameters
  //

  loadParameters();
  
  AsyncElegantOTA.begin(&server);    // Start ElegantOTA

  server.onNotFound(notFound);
  server.begin();
  Serial.println("Server started");

}


void loadParameters() {
  //
  // Start the server and wait for parameters sent by App through a GET request
  //
  

 server.on("/settings", HTTP_GET, [] (AsyncWebServerRequest *request) {

    boolean allParametersReceived = true;

    Serial.println("Receiving get request for settings");

    // Prepare the EEPROM
    Eeprom_clean();
    
    API_KEY = getParameter(request, "api_key");
    if (API_KEY.length() == 0) {
      request->send(200, "text/html", "API_KEY is wrong"); 
      
    } else if (Eeprom_write(stored_api_key_addr, stored_api_key_len, API_KEY) == false) {
      request->send(200, "text/html", "API_KEY could not be saved");  
      
    }

    USER_EMAIL = getParameter(request, "user_email");
    if (USER_EMAIL.length() == 0) {
      request->send(200, "text/html", "User email is wrong"); 
      
    } else if (Eeprom_write(stored_user_email_addr, stored_user_email_len, USER_EMAIL) == false) {
      request->send(200, "text/html", "USER_EMAIL could not be saved");  
      
    }
      
    USER_PASSWORD = getParameter(request, "user_password");
    if (USER_PASSWORD.length() == 0) {
      request->send(200, "text/html", "User password is wrong"); 
      
    } else if (Eeprom_write(stored_user_email_password_addr, stored_user_email_password_len, USER_PASSWORD) == false) {
      request->send(200, "text/html", "USER_PASSWORD could not be saved"); 
       
    }
        
    DATABASE_URL = getParameter(request, "database_url");
    if (DATABASE_URL.length() == 0) {
      request->send(200, "text/html", "DATABASE_URL is wrong"); 
      
    } else if (Eeprom_write(stored_rtdb_url_addr, stored_rtdb_url_len, DATABASE_URL) == false) {
       request->send(200, "text/html", "DATABASE_URL could not be saved");  
       
    }
     
    STORAGE_BUCKET_ID = getParameter(request, "storage_bucket_id");
    if (STORAGE_BUCKET_ID.length() == 0) {
      request->send(200, "text/html", "STORAGE_BUCKET_ID is wrong"); 
      
    } else if (Eeprom_write(stored_storage_bucket_id_addr, stored_storage_bucket_id_len, STORAGE_BUCKET_ID) == false) {
       request->send(200, "text/html", "STORAGE_BUCKET_ID could not be saved");  
       
    }
    
    USER_KEY = getParameter(request, "user_key");
    if (USER_KEY.length() == 0) {
      request->send(200, "text/html", "USER_KEY is wrong"); 
      
    } else if (Eeprom_write(stored_user_key_addr, stored_user_key_len, USER_KEY) == false) {
       request->send(200, "text/html", "USER_KEY could not be saved");  
       
    }

    SHELTER_KEY = getParameter(request, "shelter_key");
    if (SHELTER_KEY.length() == 0) {
      request->send(200, "text/html", "SHELTER_KEY is wrong"); 
      
    } else if (Eeprom_write(stored_shelter_key_addr, stored_shelter_key_len, SHELTER_KEY) == false) {
       request->send(200, "text/html", "SHELTER_KEY could not be saved");  
       
    }
         
    ssid = getParameter(request, "ssid");
    if (ssid.length() == 0) {
      request->send(200, "text/html", "ssid is wrong"); 
      
     } else if (Eeprom_write(stored_ssid_addr, stored_ssid_len, ssid) == false) {
       request->send(200, "text/html", "ssid could not be saved"); 
        
    }

    password = getParameter(request, "password");
    if (password.length() == 0) {
      request->send(200, "text/html", "password is wrong"); 
      
     } else if (Eeprom_write(stored_password_addr, stored_password_len, password) == false) {
       request->send(200, "text/html", "ssid could not be saved");  
       
    }
    
    Serial.println(API_KEY);
    Serial.println(USER_EMAIL);
    Serial.println(USER_PASSWORD);
    Serial.println(DATABASE_URL);
    Serial.println(STORAGE_BUCKET_ID);
    Serial.println(USER_KEY);
    Serial.println(USER_KEY);
    Serial.println(ssid);
    Serial.println(password);                 
    // and send acknowledgment to the App
    Serial.println("All parameters received");
    request->send(200, "text/html", "OK"); 
  });
}


String getParameter(AsyncWebServerRequest *request, String name) {
  //
  // Extract the parameter value from the http request
  //
  String value = "";
   if (request->hasParam(name)) {
    value = request->getParam(name)->value();
    Serial.println("Parameter " + name + " = " + value);
    if (value.length() == 0) {
      // Message erreur
      Serial.println("Parameter " + name + " not captured");
      request->send(200, "text/html", "Problems receiving parameter " + name);
    }; 
  } else {
    request->send(200, "text/html", "Parameter " + name + " is missing");
  }
  return value;
}


void setUpWiFiAccessPoint() {

  
  WiFi.softAP(ap_ssid, ap_password);

  IPAddress IP = WiFi.softAPIP();
  //IPAddress IP = WiFi.localIP();
  Serial.print("AP IP address: ");
  Serial.println(IP);
  
  server.begin();  
}


void notFound(AsyncWebServerRequest *request) {
  request->send(404, "text/plain", "Not found");
  Serial.println("Not found");

}
